@@ -1,4 +1,4 @@
-/*  $Id: FrenchVerbDictionary.h,v 1.37 2010/11/21 23:58:59 sarrazip Exp $
+/*  $Id: FrenchVerbDictionary.h,v 1.44 2012/04/24 02:46:05 sarrazip Exp $
     FrenchVerbDictionary.h - Dictionary of verbs and conjugation templates
 
     verbiste - French conjugation system
@@ -49,6 +49,23 @@ class FrenchVerbDictionary
 {
 public:
 
+    enum Language { NO_LANGUAGE, FRENCH, ITALIAN, GREEK };
+
+    /** Returns the language identifier recognized in the given string.
+        @param  twoLetterCode           string containing a language code
+        @returns                        a member of the 'Language' enum,
+                                        or NO_LANGUAGE if the string was
+                                        not recognized
+    */
+    static Language parseLanguageCode(const std::string &twoLetterCode);
+
+    /** Returns the two-letter code that names the given language identifier.
+        @param  l               valid language identifier
+        @returns                empty string if 'l' is NO_LANGUAGE or invalid,
+                                or a twi-letter string otherwise
+    */
+    static std::string getLanguageCode(Language l);
+
     /** Returns the full path names of the conjugation and verb XML files
         for the given language code.
         @param  conjFN          receives the full path of the conjugation template XML file
@@ -56,7 +73,7 @@ public:
         @param  languageCode    "fr" for French or "it" for Italian
     */
     static void getXMLFilenames(std::string &conjFN, std::string &verbsFN,
-                                const std::string &languageCode);
+                                Language l);
 
     /** Load a conjugation database.
         @param    conjugationFilename   filename of the XML document that
@@ -64,17 +81,16 @@ public:
         @param    verbsFilename         filename of the XML document that
                                         defines all the known verbs and their
                                         corresponding template
-        @param    includeWithoutAccents include in the knowledge base variants
+        @param    includeWithoutAccents fill knowledge base with variants of
                                         verbs where some or all accents are missing
-        @param    langCode              language of the dictionary ("fr" for French
-                                        or "it" for Italian)
+        @param    lang                  language of the dictionary
         @throws   logic_error           for invalid arguments,
                                         unparseable or unexpected XML documents
     */
     FrenchVerbDictionary(const std::string &conjugationFilename,
                         const std::string &verbsFilename,
                         bool includeWithoutAccents,
-                        const std::string &langCode)
+                        Language lang)
                                         throw (std::logic_error);
 
     /** Load the French conjugation database.
@@ -171,15 +187,15 @@ public:
     static Tense convertTenseName(const char *tenseName);
 
     /** Analyzes a conjugated verb and finds all known possible cases.
-        @param   conjugatedVerb conjugated French verb in Latin-1
-                                (e.g., "aimerions")
+        @param   utf8ConjugatedVerb     conjugated French verb in UTF-8
+                                        (e.g., "aimerions")
         @param   results        vector in which to store the inflection
                                 descriptions (this vector is not emptied
                                 before elements are stored in it);
                                 no elements are stored in this vector
                                 if the given conjugated verb is unknown
     */
-    void deconjugate(const std::string &conjugatedVerb,
+    void deconjugate(const std::string &utf8ConjugatedVerb,
                             std::vector<InflectionDesc> &results);
 
     /** Returns the English name (in ASCII) of the given mode.
@@ -190,63 +206,73 @@ public:
     */
     static const char *getTenseName(Tense t);
 
-    /** Converts a Latin-1 string to lower-case.
-        @param    latin1String  ISO-8859-1 character string to be converted
+    /** Converts the Latin-1 characters of a wide character string to lower-case.
+        @param    wideString    Unicode character string to be converted
         @returns                lower-case version of the character string
     */
-    std::string tolowerLatin1(const std::string &latin1String) const;
+    std::wstring tolowerWide(const std::wstring &wideString) const;
 
-    /** Converts a Latin-1 string into UTF-8.
-        @param    latin1String  ISO-8859-1 character string to be converted
-        @returns                UTF-8 version of the character string
-        @throws   int           errno value set by iconv(3)
+    /** Determines if a Unicode character is lower-case.
+        Only works on Latin-1 characters.
+        @param      c           Unicode character code
+        @returns                true iff 'c' is a Latin-1 vowel.
     */
-    std::string latin1ToUTF8(const std::string &latin1String) const throw(int);
+    static bool isWideVowel(wchar_t c);
 
-    /** Converts a UTF-8 string into Latin-1.
-        @param    utf8string    UTF-8 character string to be converted
-        @returns                a ISO-8859-1 version of the character string
-        @throws   int           errno value set by iconv(3)
+    /** Converts a UTF-8 string to a wide character string.
+        @param      utf8String  UTF-8 string to be converted
+        @returns                Unicode string
     */
-    std::string utf8ToLatin1(const std::string &utf8String) const throw(int);
+    std::wstring utf8ToWide(const std::string &utf8String) const throw(int);
 
-    /** Converts the strings in the designated vector from UTF-8 to Latin-1.
-        @param    vec           structure whose strings are to be converted
-        @throws   int           errno value set by iconv(3)
+    /** Converts a wide character string to a UTF-8 string.
+        @param      wideString  Unicode string to be converted
+        @returns                UTF-8 string
     */
-    void utf8ToLatin1(std::vector<InflectionDesc> &vec) const throw(int);
-
-    /** Converts the strings in the designated vector from UTF-8 to Latin-1.
-        @param    vec           structure whose strings are to be converted
-        @throws   int           errno value set by iconv(3)
-    */
-    void utf8ToLatin1(
-                std::vector<std::vector<std::string> > &vec) const throw(int);
+    std::string wideToUTF8(const std::wstring &wideString) const throw(int);
 
     /** Removes accents from accented letters in the given string.
-        @param   latin1String   ISO-8859-1 string with accented characters
-        @returns                a Latin-1 string with the accents removed
+        @param   utf8String     UTF-8 string with accented characters
+        @returns                a UTF-8 string with the accents removed
     */
-    static std::string removeAccents(const std::string &latin1String);
+    std::string removeUTF8Accents(const std::string &utf8String);
 
-    /** Returns all unaccented variants of a string.
-        If N letters are accented in 'latin1String', then 2^N variants
+    /** Returns all unaccented variants of a wide character string.
+        If N letters are accented in 'utf8String', then 2^N variants
         are returned, where each accented letter either keeps or loses
         its accents.
         For example, the word "été" has 3 unaccented variants:
         "eté", "ete" and "éte".
-        @param    latin1String  ISO-8859-1 string with accented characters
+        @param    wideString    wide character string with accented characters
         @param    index         pass zero (recursive calls to this function
                                 pass non-zero indices)
-        @param    variants      vector to which Latin-1 strings are added
+        @param    utf8Variants  vector to which UTF-8 strings are added
                                 with push_back (this function does not
                                 clear the vector beforehand)
     */
-    static void formUnaccentedVariants(const std::string &latin1String,
+    void formUTF8UnaccentedVariants(const std::wstring &wideString,
                                         size_t index,
-                                        std::vector<std::string> &variants);
+                                        std::vector<std::string> &utf8Variants);
 
-    /**        Returns the content of an XML node in Latin-1.
+    /** Returns all unaccented variants of a UTF-8 string.
+        If N letters are accented in 'utf8String', then 2^N variants
+        are returned, where each accented letter either keeps or loses
+        its accents.
+        For example, the word "été" has 3 unaccented variants:
+        "eté", "ete" and "éte".
+        @param    utf8String    UTF-8 string with accented characters
+        @param    index         pass zero (recursive calls to this function
+                                pass non-zero indices)
+        @param    utf8Variants  vector to which UTF-8 strings are added
+                                with push_back (this function does not
+                                clear the vector beforehand)
+    */
+    void formUTF8UnaccentedVariants(const std::string &utf8String,
+                                        size_t index,
+                                        std::vector<std::string> &utf8Variants);
+
+
+    /** Returns the content of an XML node in UTF-8.
         @param    doc           the XML document
         @param    node          the node of the XML document whose contents
                                 are to be extracted
@@ -256,10 +282,12 @@ public:
         @throws    int          errno value set by iconv(3), in the case of a
                                 UTF-8 to Latin-1 conversion error
     */
-    std::string getLatin1XmlNodeText(
+    std::string getUTF8XmlNodeText(
                         xmlDocPtr doc, xmlNodePtr node) throw(int);
 
-    /**        Returns the content of an XML property in Latin-1.
+    /** Returns the content of an XML property in UTF-8.
+        For example, if 'node' represents <foo type='xyz'/>,
+        then passing "type" for 'propName' will return "xyz".
         @param    node          the node of the XML document
         @param    propName      the name of the property to extract
         @returns                a Latin-1 string representing the contents
@@ -268,9 +296,10 @@ public:
         @throws   int           errno value set by iconv(3), in the case of a
                                 UTF-8 to Latin-1 conversion error
     */
-    std::string getLatin1XmlProp(
+    std::string getUTF8XmlProp(
                         xmlNodePtr node, const char *propName) throw(int);
 
+
     /** Gets the radical part of an infinitive, according to a template name.
         @param    infinitive    infinitive whose radical is requested
         @param    templateName  name of the conjugation template that applies
@@ -301,14 +330,14 @@ public:
         @returns                true for success, or false if the mode or
                                 tense is unknown.
     */
-    static bool generateTense(const std::string &radical,
+    bool generateTense(const std::string &radical,
                         const TemplateSpec &templ,
                         Mode mode,
                         Tense tense,
                         std::vector< std::vector<std::string> > &dest,
                         bool includePronouns,
                         bool aspirateH,
-                        bool isItalian) throw();
+                        bool isItalian) const throw();
 
     /** Indicates if the given verb starts with an aspirate h.
         An aspirate h means that one cannot make a contraction or liaison
@@ -320,10 +349,9 @@ public:
     bool isVerbStartingWithAspirateH(
                                 const std::string &infinitive) const throw();
 
-    /** Returns the two-letter code representing this dictionary's language.
-        @returns                    "fr" for French, "it" for Italian.
+    /** Returns the code representing this dictionary's language.
     */
-    std::string getLanguageCode() const { return langCode; }
+    Language getLanguage() const { return lang; }
 
 private:
 
@@ -365,6 +393,7 @@ private:
         }
 
         /** Callback invoked by the Trie<>::get() method.
+            Inherited from Trie<>.
             This callback will be called for each prefix of the searched
             string that corresponds to the radical of a known verb.
             Stores data in the vector<InflectionDesc> designated by
@@ -375,8 +404,8 @@ private:
                                             might apply to the conjugated verb
         */
         virtual void onFoundPrefixWithUserData(
-                        const std::string &conjugatedVerb,
-                        std::string::size_type index,
+                        const std::wstring &conjugatedVerb,
+                        std::wstring::size_type index,
                         const std::vector<TrieValue> *templateList) const
                                                                 throw();
 
@@ -408,11 +437,11 @@ private:
     VerbTable knownVerbs;
     std::set<std::string> aspirateHVerbs;
     InflectionTable inflectionTable;
-    iconv_t toUTF8;
-    iconv_t toLatin1;
+    iconv_t wideToUTF8Conv;
+    iconv_t utf8ToWideConv;
     char latin1TolowerTable[256];
     VerbTrie verbTrie;
-    std::string langCode;  // two-letter language code (e.g., "fr")
+    Language lang;
 
 private:
 
