@@ -1,4 +1,4 @@
-/*  $Id: Trie.cpp,v 1.9 2010/07/11 02:56:15 sarrazip Exp $
+/*  $Id: Trie.cpp,v 1.11 2011/01/08 19:07:36 sarrazip Exp $
     Trie.cpp - Tree structure for string storage
 
     verbiste - French conjugation system
@@ -23,7 +23,9 @@
 #include "Trie.h"
 
 #include <assert.h>
+#include <stdlib.h>
 #include <list>
+#include <iostream>
 
 
 namespace verbiste {
@@ -90,9 +92,6 @@ template <class T>
 size_t
 Trie<T>::CharDesc::computeMemoryConsumption() const
 {
-    // Note: character member 'c' is typically followed by padding before 'desc' starts.
-    // We use the following subtraction to get the size of 'c' with its padding.
-    //
     return (sizeof(*this) - sizeof(desc)) + desc.computeMemoryConsumption();
 }
 
@@ -121,11 +120,11 @@ Trie<T>::Row::recursiveDelete(bool deleteUserData)
 
 template <class T>
 typename Trie<T>::Descriptor *
-Trie<T>::Row::find(unsigned char c)
+Trie<T>::Row::find(wchar_t unichar)
 {
     for (typename std::vector<CharDesc>::iterator it = elements.begin();
                                         it != elements.end(); it++)
-        if (it->c == c)
+        if (it->unichar == unichar)
             return &it->desc;
 
     return NULL;
@@ -134,13 +133,14 @@ Trie<T>::Row::find(unsigned char c)
 
 template <class T>
 typename Trie<T>::Descriptor &
-Trie<T>::Row::operator [] (unsigned char c)
+Trie<T>::Row::operator [] (wchar_t unichar)
 {
-    Descriptor *pd = find(c);
+    Descriptor *pd = find(unichar);
     if (pd != NULL)
         return *pd;
 
-    elements.push_back(CharDesc(c));
+    elements.push_back(CharDesc(unichar));
+    assert(elements.back().unichar == unichar);
     return elements.back().desc;
 }
 
@@ -150,7 +150,7 @@ Trie<T>::Row::operator [] (unsigned char c)
 
 template <class T>
 T *
-Trie<T>::add(const std::string &key, T *userData)
+Trie<T>::add(const std::wstring &key, T *userData)
 {
     if (key.empty())
     {
@@ -169,7 +169,7 @@ Trie<T>::add(const std::string &key, T *userData)
 
 template <class T>
 T *
-Trie<T>::get(const std::string &key) const
+Trie<T>::get(const std::wstring &key) const
 {
     if (lambda != NULL)
         onFoundPrefixWithUserData(key, 0, lambda);
@@ -184,7 +184,7 @@ Trie<T>::get(const std::string &key) const
 
 template <class T>
 T *
-Trie<T>::getWithDefault(const std::string &key, T *deFault)
+Trie<T>::getWithDefault(const std::wstring &key, T *deFault)
 {
     if (key.empty())
     {
@@ -203,7 +203,7 @@ Trie<T>::getWithDefault(const std::string &key, T *deFault)
 
 template <class T>
 T **
-Trie<T>::getUserDataPointer(const std::string &key)
+Trie<T>::getUserDataPointer(const std::wstring &key)
 {
     if (key.empty())
         return &lambda;
@@ -220,42 +220,64 @@ Trie<T>::getUserDataPointer(const std::string &key)
 template <class T>
 typename Trie<T>::Descriptor *
 Trie<T>::getDesc(Row *row,
-                const std::string &key,
-                std::string::size_type index,
+                const std::wstring &key,
+                std::wstring::size_type index,
                 bool create,
                 bool callFoundPrefixCallback)
 {
+    assert(row != NULL);
     assert(index < key.length());
-    unsigned char c = (unsigned char) key[index];  // the "expected" character
-    Descriptor *pd = row->find(c);
+
+    wchar_t unichar = key[index];  // the "expected" character
+    assert(unichar != '\0');
+
+    Descriptor *pd = row->find(unichar);
+
+    static bool trieTrace = getenv("TRACE") != NULL;
+    if (trieTrace)
+        std::wcout << "getDesc(row=" << row
+                   << ", key='" << key << "' (len=" << key.length()
+                   << "), index=" << index
+                   << ", create=" << create
+                   << ", call=" << callFoundPrefixCallback
+                   << "): unichar=" << unichar << ", pd=" << pd << "\n";
 
     if (pd == NULL)  // if expected character not found
     {
         if (!create)
             return NULL;
 
-        Descriptor &newDesc = (*row)[c];
+        Descriptor &newDesc = (*row)[unichar];
+        assert(row->find(unichar) != NULL);
+        assert(row->find(unichar) == &newDesc);
 
         if (index + 1 == key.length())  // if last char of string
             return &newDesc;
 
         // Create new descriptor that points to a new inferior row:
         newDesc.inferiorRow = new Row();
+        assert(row->find(unichar)->inferiorRow == newDesc.inferiorRow);
+
         return getDesc(newDesc.inferiorRow,
                         key, index + 1, create, callFoundPrefixCallback);
     }
 
+    if (trieTrace)
+        std::wcout << "getDesc: userData=" << pd->userData
+                   << ", inferiorRow=" << pd->inferiorRow
+                   << "\n";
+
+    if (callFoundPrefixCallback && pd->userData != NULL)
+        onFoundPrefixWithUserData(key, index + 1, pd->userData);  // virtual call
+
     if (index + 1 == key.length())  // if reached end of key
     {
-        if (callFoundPrefixCallback && pd->userData != NULL)
-            onFoundPrefixWithUserData(key, index + 1, pd->userData);
+        if (trieTrace)
+            std::wcout << "getDesc: reached end of key\n";
         return pd;
     }
 
-    if (callFoundPrefixCallback && pd->userData != NULL)
-        onFoundPrefixWithUserData(key, index + 1, pd->userData);
-
-    if (pd->inferiorRow == NULL)  // if d is a leaf:
+    if (pd->inferiorRow == NULL)  // if pd is a leaf:
     {
         if (!create)
             return NULL;  // not found
