@@ -1,4 +1,4 @@
-/*  $Id: misc-types.h,v 1.10 2010/07/04 05:49:51 sarrazip Exp $
+/*  $Id: misc-types.h,v 1.12 2011/01/26 02:03:28 sarrazip Exp $
     misc-types.h - Miscellaneous types used by the dictionary class.
 
     verbiste - French conjugation system
@@ -45,6 +45,17 @@ enum Mode
     IMPERATIVE_MODE = VERBISTE_IMPERATIVE_MODE,
     PARTICIPLE_MODE = VERBISTE_PARTICIPLE_MODE,
     GERUND_MODE = VERBISTE_GERUND_MODE,
+
+    // Greek modes:
+    PRESENT_INDICATIVE = VERBISTE_PRESENT_INDICATIVE,
+    PRESENT_SUBJUNCTIVE = VERBISTE_PRESENT_SUBJUNCTIVE,
+    PRESENT_IMPERATIVE = VERBISTE_PRESENT_IMPERATIVE,
+    PRESENT_GERUND = VERBISTE_PRESENT_GERUND,
+    PAST_IMPERFECT_INDICATIVE = VERBISTE_PAST_IMPERFECT_INDICATIVE,
+    PAST_PERFECT_INDICATIVE = VERBISTE_PAST_PERFECT_INDICATIVE,
+    PAST_PERFECT_SUBJUNCTIVE = VERBISTE_PAST_PERFECT_SUBJUNCTIVE,
+    PAST_PERFECT_IMPERATIVE = VERBISTE_PAST_PERFECT_IMPERATIVE,
+    PAST_PERFECT_INFINITIVE = VERBISTE_PAST_PERFECT_INFINITIVE
 };
 
 
@@ -57,9 +68,15 @@ enum Tense
     PRESENT_TENSE = VERBISTE_PRESENT_TENSE,
     PAST_TENSE = VERBISTE_PAST_TENSE,
     IMPERFECT_TENSE = VERBISTE_IMPERFECT_TENSE,
-    FUTURE_TENSE = VERBISTE_FUTURE_TENSE
-};
+    FUTURE_TENSE = VERBISTE_FUTURE_TENSE,
 
+    // Greek tenses:
+    ACTIVE_TENSE = VERBISTE_ACTIVE_TENSE,
+    PASSIVE_TENSE = VERBISTE_PASSIVE_TENSE,
+    IMPERATIVE_ACTIVE_TENSE = VERBISTE_IMPERATIVE_ACTIVE_TENSE,
+    IMPERATIVE_PASSIVE_TENSE = VERBISTE_IMPERATIVE_PASSIVE_TENSE,
+    PAST_PERFECT = VERBISTE_PAST_PERFECT
+};
 
 
 /**
